@@ -488,7 +488,7 @@ dma_sparse_view_populate_popup (DmaSparseView *view,
 	gtk_widget_show (menu_item);
 
 	/* create goto menu_item. */
-	menu_item = gtk_menu_item_new_with_mnemonic ("_Goto address");
+	menu_item = gtk_menu_item_new_with_mnemonic (_("_Goto address"));
 	g_signal_connect (G_OBJECT (menu_item), "activate",
 			  G_CALLBACK (dma_sparse_view_goto_activate), view);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
@@ -546,11 +546,11 @@ dma_sparse_view_synchronize_iter (DmaSparseView *view, DmaSparseIter *iter)
 
 	if (dist != 0)
 	{
-		gint count = (gint)(dist / view->priv->vadjustment->step_increment);
-		
-		if ((count < 2.0 * (view->priv->vadjustment->page_size))
-			 && (count > -2.0 * (view->priv->vadjustment->page_size)))
+		if ((dist < 4.0 * (view->priv->vadjustment->page_size))
+			 && (dist > -4.0 * (view->priv->vadjustment->page_size)))
         {
+			gint count = (gint)(dist / view->priv->vadjustment->step_increment);
+			
         	dma_sparse_iter_forward_lines (iter, count);
         }
         else
@@ -944,6 +944,26 @@ dma_sparse_view_goto (DmaSparseView *view, guint location)
 	gtk_adjustment_value_changed (view->priv->vadjustment);
 }
 
+guint
+dma_sparse_view_get_location (DmaSparseView *view)
+{
+	GtkTextMark *mark;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	DmaSparseIter buf_iter;
+	gint line;
+	
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(view));
+	mark = gtk_text_buffer_get_insert (buffer);
+	gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
+	line = gtk_text_iter_get_line (&iter);
+		
+	dma_sparse_iter_copy (&buf_iter, &view->priv->start);
+	dma_sparse_iter_forward_lines (&buf_iter, line);
+	
+	return dma_sparse_iter_get_address (&buf_iter);
+}
+
 /* GtkWidget functions
  *---------------------------------------------------------------------------*/
 
