@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef _THREADS_H_
@@ -32,7 +32,7 @@
 
 typedef struct _DmaThreads DmaThreads;
 
-DmaThreads *dma_threads_new (IAnjutaDebugger *debugger, DebugManagerPlugin *plugin);
+DmaThreads *dma_threads_new (DebugManagerPlugin *plugin);
 void dma_threads_free (DmaThreads *self);
 
 #endif
