@@ -815,14 +815,17 @@ static gint
 get_line_indentation_base (CppJavaPlugin *plugin,
 						   IAnjutaEditor *editor,
 						   gint line_num,
-						   gint *incomplete_statement)
+						   gint *incomplete_statement,
+						   gboolean *colon_indent)
 {
 	IAnjutaIterable *iter;
 	gchar point_ch;
 	gint line_indent = 0;
+	gint extra_indent = 0;
 	gboolean looking_at_just_next_line = TRUE;
 	gboolean current_line_is_preprocessor = FALSE;
 	gboolean current_line_is_continuation = FALSE;
+	gboolean line_checked_for_comment = FALSE;
 	
 	*incomplete_statement = -1;
 	
@@ -858,16 +861,114 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	
 	while (ianjuta_iterable_previous (iter, NULL))
 	{
-		/* Skip comments and strings */
+		/* Skip strings */
 		IAnjutaEditorAttribute attrib =
 			ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter), NULL);
-		if (attrib == IANJUTA_EDITOR_COMMENT || attrib == IANJUTA_EDITOR_STRING)
+		if (attrib == IANJUTA_EDITOR_STRING)
 			continue;
 		
 		point_ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0,
 												 NULL);
 
 		/* DEBUG_PRINT("point_ch = %c", point_ch); */
+		/* Check if we are inside a comment */
+		if (!line_checked_for_comment && !isspace(point_ch))
+		{
+			gboolean comment = FALSE;
+			gboolean comment_end = FALSE;
+			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
+			do
+			{
+				gchar c;
+				c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter), 0,
+												  NULL);
+				if (!comment_end && iter_is_newline (new_iter, c))
+				{
+					line_checked_for_comment = TRUE;
+					break;
+				}
+				if (c == '/')
+				{
+					ianjuta_iterable_previous (new_iter, NULL);
+					c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter), 0,
+													  NULL);
+					if (c == '/')
+					{
+						/* is a line comment, skip until begin of comment */
+						comment = TRUE;
+						break;
+					}
+				}
+				if (c == '*')
+				{
+					IAnjutaIterable* prev = ianjuta_iterable_clone (new_iter, NULL);
+					IAnjutaIterable* next = ianjuta_iterable_clone (new_iter, NULL);					
+					ianjuta_iterable_previous (prev, NULL);
+					ianjuta_iterable_next (next, NULL);
+					gchar prev_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (prev), 0,
+													  NULL);
+					gchar next_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (next), 0,
+													  NULL);					
+					if (prev_c == '/')
+					{
+						/* starts comment */
+						comment = TRUE;
+						DEBUG_PRINT ("Starts comment");
+						if (!comment_end)
+							extra_indent++;
+						g_object_unref (prev);
+						g_object_unref (next);
+						break;
+					}
+					else if (next_c == '/')
+					{
+						/* ends comment */
+						DEBUG_PRINT ("Ends comment");
+						comment_end = TRUE;
+						g_object_unref (prev);
+						g_object_unref (next);
+						continue;
+					}
+					/* Possibly continued comment */
+					else if (isspace(prev_c))
+					{
+						gboolean possible_comment = FALSE;
+						while (ianjuta_iterable_previous (prev, NULL))
+						{
+							prev_c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (prev), 0,
+															  NULL);
+							if (!isspace(prev_c))
+								break;
+							if (iter_is_newline (prev, prev_c))
+							{
+								possible_comment = TRUE;
+								break;
+							}
+						}
+						if (possible_comment)
+						{
+							ianjuta_iterable_set_position (new_iter, 
+														   ianjuta_iterable_get_position (prev, NULL),
+														   NULL);
+							g_object_unref (prev);
+							g_object_unref (next);
+							continue;
+						}
+					}
+					g_object_unref (prev);
+					g_object_unref (next);
+				}
+			} while (ianjuta_iterable_previous (new_iter, NULL));
+			if (comment)
+			{
+				ianjuta_iterable_set_position (iter,
+											   ianjuta_iterable_get_position (new_iter, NULL) - 1, 
+											   NULL);
+				g_object_unref (new_iter);
+				continue;
+			}
+			g_object_unref (new_iter);
+		}
 		if (point_ch == ')' || point_ch == ']' || point_ch == '}')
 		{
 			gint line_saved;
@@ -887,6 +988,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (point_ch == '}' && get_line_indentation (editor, line_saved) <= 0)
 			{
 				line_indent = 0;
+				line_indent += extra_indent;
 				break;
 			}
 			
@@ -894,9 +996,10 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (!cpp_java_util_jump_to_matching_brace (iter, point_ch, -1))
 			{
 				line_indent = get_line_indentation (editor, line_saved);
+				line_indent += extra_indent;
 				break;
 			}
-		}
+			}
 		else if (point_ch == '{')
 		{
 			gint line_for_indent =
@@ -904,6 +1007,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			line_indent = get_line_indentation (editor, line_for_indent);
 			/* Increase line indentation */
 			line_indent += INDENT_SIZE;
+			line_indent += extra_indent;
 			
 			/* If we encounter a block-start before anything else, the
 			 * statement could hardly be incomplte.
@@ -931,6 +1035,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 					line_indent++;
 			}
 			line_indent += 1;
+			line_indent += extra_indent;
 			
 			/* Although statement is incomplete at this point, we don't
 			 * set it to incomplete and just leave it to unknown to avaoid
@@ -948,6 +1053,37 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			if (*incomplete_statement == -1)
 				*incomplete_statement = 0;
 		}
+		else if (point_ch == ':' && *colon_indent == FALSE)
+		{
+			/* This is a forward reference, all lines below should have
+			 * increased indentation until the next statement has
+			 * a ':'
+			 */
+			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
+			gboolean indent = FALSE;		
+			
+			/* Is the last non-whitespace in line */
+			while (ianjuta_iterable_next (new_iter, NULL))
+			{
+				gchar c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter),
+													   0, NULL);
+				if (!isspace(c))
+					break;
+				if (iter_is_newline (new_iter, c))
+				{
+					indent = TRUE;
+					break;
+				}
+			}
+			g_object_unref (new_iter);
+			if (indent)
+			{
+				*colon_indent = TRUE;
+				extra_indent += INDENT_SIZE;
+				if (*incomplete_statement == -1)
+					*incomplete_statement = 0;
+			}
+		}	
 		else if (iter_is_newline (iter, point_ch))
 		{
 			skip_iter_to_newline_head (iter, point_ch);
@@ -965,9 +1101,10 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				*/
 			}
 			looking_at_just_next_line = FALSE;
-		}
+			line_checked_for_comment = FALSE;
+		}		 
 		else if (!isspace (point_ch))
-		{
+		{			
 			/* If we encounter any non-whitespace char before any of the
 			 * statement-complete indicators, the statement is basically
 			 * incomplete
@@ -976,47 +1113,47 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				*incomplete_statement = 1;
 		}
 	}
+	if (ianjuta_iterable_first (iter, NULL))
+		line_indent += extra_indent;
 	g_object_unref (iter);
 	
 	return line_indent;
 }
 
-/* Check if iter is inside comment or string. Begining of comment or string
+/* Check if iter is inside string. Begining of string
  * is not counted as inside.
  */
 static gboolean
-is_iter_inside_comment_or_string (IAnjutaIterable *iter)
+is_iter_inside_string (IAnjutaIterable *iter)
 {
 	IAnjutaEditorAttribute attrib;
 	
 	attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
 												NULL);
-	/* Check if we are *inside* comment or string. Begining of comment
-	 * or string does not count as inside.
+	/* Check if we are *inside* string. Begining
+	 * of string does not count as inside.
 	 */
-	if (attrib == IANJUTA_EDITOR_COMMENT || attrib == IANJUTA_EDITOR_STRING)
+	if (attrib == IANJUTA_EDITOR_STRING)
 	{
 		/* Peek previous attrib and see what it was */
 		if (ianjuta_iterable_previous (iter, NULL))
 		{
 			attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL
 														(iter),	NULL);
-			if (attrib == IANJUTA_EDITOR_COMMENT ||
-				attrib == IANJUTA_EDITOR_STRING)
+			if (attrib == IANJUTA_EDITOR_STRING)
 			{
-				/* We are inside comment or string */
+				/* We are inside string */
 				return TRUE;
 			}
 			else
 			{
-				/* The comment or string just began, not inside.
+				/* The string just began, not inside.
 				 * Restore iter from the peek
 				 */
 				ianjuta_iterable_next (iter, NULL);
 			}
 		}
-		/* else, there is no previous and so we can't be inside comment
-		 * or string
+		/* else, there is no previous and so we can't be inside string
 		 */
 	}
 	return FALSE;
@@ -1027,8 +1164,10 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 						   gint line)
 {
 	IAnjutaIterable *iter;
+	IAnjutaIterable *end_iter;
 	gint line_indent = 0;
 	gint incomplete_statement = -1;
+	gboolean colon_indent = FALSE;
 	
 	g_return_val_if_fail (line > 0, 0);
 	
@@ -1037,14 +1176,35 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	
 	iter = ianjuta_editor_get_line_begin_position (editor, line, NULL);
 	
-	if (is_iter_inside_comment_or_string (iter))
+	if (is_iter_inside_string (iter))
 	{
 		line_indent = get_line_indentation (editor, line - 1);
 	}
 	else
 	{
 		line_indent = get_line_indentation_base (plugin, editor, line,
-												 &incomplete_statement);
+												 &incomplete_statement, &colon_indent);
+	}
+	
+	if (colon_indent)
+	{
+		/* If the last non-whitespace character in the line is ":" then
+		 * we remove the extra colon_indent
+		 */
+		end_iter = ianjuta_editor_get_line_end_position (editor, line, NULL);
+		gchar ch;
+		do
+		{
+			ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (end_iter),
+											   0, NULL);
+			if (ch == ':')
+			{
+				line_indent -= INDENT_SIZE;
+				break;
+			}
+		}
+		while (isspace(ch) && ianjuta_iterable_previous (end_iter, NULL));
+		g_object_unref (end_iter);
 	}
 	
 	/* Determine what the first non-white char in the line is */
@@ -1055,7 +1215,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		 * or string does not count as inside. If inside, just align with
 		 * previous indentation.
 		 */
-		if (is_iter_inside_comment_or_string (iter))
+		if (is_iter_inside_string (iter))
 		{
 			line_indent = get_line_indentation (editor, line - 1);
 			break;
@@ -1083,6 +1243,9 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 							 line_indent + BRACE_INDENT);
 				*/
 				line_indent += BRACE_INDENT;
+				/* It looks ugly to add extra indent after case: so remove that */
+				if (colon_indent)
+					line_indent -= INDENT_SIZE;
 			}
 			break;
 		}
@@ -1143,11 +1306,10 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 	{
 		/* Indent only when it's the first non-white space char in the line */
 		
-		/* Don't bother if we are inside comment or string */
+		/* Don't bother if we are inside string */
 		attrib = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
 													NULL);
-		if (attrib != IANJUTA_EDITOR_COMMENT &&
-			attrib != IANJUTA_EDITOR_STRING)
+		if (attrib != IANJUTA_EDITOR_STRING)
 		{
 			/* Iterate backwards till the begining of the line and disable
 			 * indenting if any non-white space char is encountered
@@ -1178,6 +1340,10 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 			}
 		}
 	}
+	else if (ch == ':')
+	{
+		should_auto_indent = TRUE;
+	}
 	if (should_auto_indent)
 	{
 		gint insert_line;
@@ -1409,7 +1575,7 @@ on_auto_indent (GtkAction *action, gpointer data)
 	{
 		line_indent = get_line_auto_indentation (lang_plugin, editor,
 												 insert_line);
-		DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent);
+		/* DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent); */
 		set_line_indentation (editor, insert_line, line_indent);
 	}
 	ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
